# -*- encoding: utf-8 -*-

from django.contrib.syndication.views import Feed
from django.utils.feedgenerator import Atom1Feed

from django_pages.pages.models import Post
from django_pages.feed.models import FeedSettings


class RssLatestPostsFeed(Feed):

    link = '/'

    try:

        settings = FeedSettings.objects.get(pk=1)

        if not settings.active:

            title = "Feeds are disabled for this site."
            description = ""
            post_count = 0

        else:

            title = settings.site_title
            description = settings.site_description
            post_count = int(settings.latest_post_count)

    except FeedSettings.DoesNotExist:

        title = "No FeedSettings found, create one in admin"
        description = "Please create exactly one feed setting in site administration"
        post_count = 5

    def items(self):
        return Post.objects.filter(active=True).order_by('-created')[:self.post_count]

    def item_title(self, item):
        return item.title

    def item_description(self, item):
        return '{} ...'.format(item.content[:200])

    def item_link(self, item):
        return item.get_url()


class AtomLatestPostsFeed(RssLatestPostsFeed):

    feed_type = Atom1Feed



import os
import stat
from subprocess import call

from django_pages.connector import ElementTree
from django_pages.connector.settings import IGNORE_FOLDERS


def get_folders(xml_response, folder_path):
    """Add information about the folders in [folder_path] to
    [xml_response].
    """

    folders = ElementTree.Element("Folders")

    for f in os.listdir(folder_path):
        if os.path.isdir(os.path.join(folder_path, f)) and not f in IGNORE_FOLDERS:
            folders.append(ElementTree.Element("Folder", {'name': f}))
    xml_response.getroot().append(folders)


def get_folders_and_files(xml_response, folder_path):

    folders = ElementTree.Element("Folders")
    files = ElementTree.Element("Files")

    for f in os.listdir(folder_path):
        if os.path.isdir(os.path.join(folder_path, f)):
            if not f in IGNORE_FOLDERS:
                folders.append(ElementTree.Element("Folder", {'name': f}))
        else:
            size = os.lstat(os.path.join(folder_path, f))[stat.ST_SIZE]
            size = str(size / 1024)

            files.append(ElementTree.Element("File",
                                             {'name': f,
                                              'size': size}
                                             )
                         )

    xml_response.getroot().append(folders)
    xml_response.getroot().append(files)


def file_upload(request, folder_path):
    """Handle a file upload and return the status code."""

    new_file = request.FILES.get('NewFile', None)
    if new_file is None:
        status = "202"
    else:
        # determine the destination file name
        file_name = new_file.name
        base, ext = os.path.splitext(file_name)
        count = 1

        while (os.path.exists(os.path.join(folder_path, file_name))):
            file_name = '%s(%s)%s' % (base, count, ext)
            count += 1

        # write the file
        target = file(os.path.join(folder_path, file_name), 'wb')
        for chunk in new_file.chunks():
            target.write(chunk)
        target.close()

        file_w_path = os.path.join(folder_path, file_name)

        # call resizer for file
        call(['convert', file_w_path, '-resize', '800x600', file_w_path])

        # set the status
        if file_name == new_file.name:
            status = "0"
        else:
            status = "201"

        return status, file_name



# -*- encoding: utf-8 -*-

from django.db import models


class FeedSettings(models.Model):
    '''
    Stores Rss/Atom feed settings
    '''

    active = models.BooleanField('Feed enabled', default=True)
    site_title = models.CharField('Site Feed title', max_length=200)
    site_description = models.TextField('Site Feed description', max_length=1000)
    latest_post_count = models.IntegerField('Latest Post count to show', default=10)

    def __unicode__(self):

        return self.site_title

    class Meta():

        verbose_name_plural = 'Feed Settings'



# -*- encoding: utf-8 -*-


def activate(modeladmin, request, queryset):

    queryset.update(active=True)

activate.short_description = 'Activate selected items.'


def deactivate(modeladmin, request, queryset):

    queryset.update(active=False)

deactivate.short_description = 'Deactivate selected items.'



# -*- encoding: utf-8 -*-

from django.db import models


class Site(models.Model):

    domain = models.CharField('Domain', max_length=150)
    display_name = models.CharField('Display name', max_length=200)
    tagline = models.CharField('Tagline', max_length=200, blank=True)
    footer = models.TextField('Footer', max_length=1000, blank=True)

    def __unicode__(self):

        return self.domain



# -*- encoding: utf-8 -*-

from django.db import models


class Log(models.Model):
    '''
    Stores request logs
    '''

    when = models.DateTimeField('When')
    logline = models.TextField('Log', max_length=10000)

    def __unicode__(self):

        return self.when.isoformat()



# -*- encoding: utf-8 -*-

import reversion


class SiteAdmin(reversion.VersionAdmin):

    list_display = ('domain', 'display_name', 'tagline')

    class Media:

        js = [
            '/static/admin/js/ckeditor/ckeditor.js',
            '/static/admin/js/ckeditor/config.js',
            '/static/admin/js/ckeditor/start.js',
        ]

        css = {
            'ckeditor': ('/static/admin/js/ckeditor/contents.css',),
        }



from django.conf.urls import patterns, include, url
from django.contrib import admin

admin.autodiscover()

from django.views.generic import TemplateView

from django_pages.feed.feed import RssLatestPostsFeed, AtomLatestPostsFeed

urlpatterns = patterns(
    '',
    # admin part
    url(r'^admin/doc/', include('django.contrib.admindocs.urls')),
    url(r'^admin/', include(admin.site.urls)),

    # ckeditor image upload connector
    url(r'^connector/browser/$', 'django_pages.connector.views.browser'),
    url(r'^connector/uploader/$', 'django_pages.connector.views.uploader'),

    # helper site (wizzard)
    url(r'^wizzard/$', TemplateView.as_view(template_name='wizzard.html')),

    # rss and atom feed
    url(r'^rss/', RssLatestPostsFeed()),
    url(r'^atom/', AtomLatestPostsFeed()),

    # robots.txt
    url(r'^robots.txt$', 'django_pages.views.robots'),
    # sitemap generator
    url(r'^sitemap.xml$', 'django_pages.views.generate_sitemap'),
    # django_pages url resolver
    url(r'^(?P<url>.*)$', 'django_pages.views.main_view'),
)



# -*- encoding: utf-8 -*-

import string

from django import forms
from django.conf.urls import patterns
from django.contrib import admin
from django.core.exceptions import PermissionDenied
from django.shortcuts import redirect, get_object_or_404
from django.utils.translation import ugettext as _
from django.utils.translation import ugettext_lazy

import reversion

from django_pages.menu.models import MenuItem
from django_pages.settings import ADMIN_MEDIA_PREFIX


class MenuItemInline(admin.TabularInline):

    model = MenuItem


class MenuItemUrlField(forms.CharField):

    def validate(self, value):

        super(MenuItemUrlField, self).validate(value)

        allowed = '%s%s%s' % (string.ascii_letters, string.digits, '-._')

        for letter in value:

            if letter not in allowed:

                raise forms.ValidationError(u'These characters are allowed: %s' % allowed)


class MenuItemAdminForm(forms.ModelForm):

    url = MenuItemUrlField('Url', min_length=4)

    class Meta:

        model = MenuItem


class MenuItemAdmin(reversion.VersionAdmin):

    form = MenuItemAdminForm
    fields = (('lang', 'menuitem_name'), 'url')
    list_display = ('menuitem_name', 'lang', 'url', 'move', 'position')

    def move(sefl, obj):
        '''
        Returns html with links to move_up and move_down views.
        '''

        button = u'<a href="%s"><img src="%simg/arrow-%s.png" /> %s</a>'
        prefix = ADMIN_MEDIA_PREFIX

        link = '%d/move_up/' % obj.pk
        html = button % (link, prefix, 'up', _('up')) + " | "
        link = '%d/move_down/' % obj.pk
        html += button % (link, prefix, 'down', _('down'))
        return html

    move.allow_tags = True
    move.short_description = ugettext_lazy('Move')

    def get_urls(self):

        admin_view = self.admin_site.admin_view

        urls = patterns(
            '',
            (r'^(?P<item_pk>\d+)/move_up/$', admin_view(self.move_up)),
            (r'^(?P<item_pk>\d+)/move_down/$', admin_view(self.move_down)),
        )

        return urls + super(MenuItemAdmin, self).get_urls()

    def move_up(self, request, item_pk):

        if self.has_change_permission(request):

            item = get_object_or_404(MenuItem, pk=item_pk)

            item.increase_position()

        else:

            raise PermissionDenied

        return redirect('/admin/menu/menuitem/?o=2.-5')

    def move_down(self, request, item_pk):

        if self.has_change_permission(request):

            item = get_object_or_404(MenuItem, pk=item_pk)

            item.decrease_position()

        else:

            raise PermissionDenied

        return redirect('/admin/menu/menuitem/?o=2.-5')



# -*- encoding: utf-8 -*-

from django import forms


class CommentForm(forms.Form):

    user = forms.CharField(max_length=100)
    comment = forms.CharField(max_length=1000, widget=forms.Textarea)

    one = forms.BooleanField(required=False)
    two = forms.BooleanField(required=False)
    three = forms.BooleanField(required=False)
    four = forms.BooleanField(required=False)



# -*- encoding: utf-8 -*-

from django.contrib import admin

from django_pages.comments.models import Comment
from django_pages.feed.models import FeedSettings
from django_pages.language.models import Language
from django_pages.log.models import Log
from django_pages.menu.models import MenuItem
from django_pages.metadata.models import MetaSet, MetaData
from django_pages.pages.models import Page, Post
from django_pages.site.models import Site

from django_pages.comments.admin import CommentAdmin
from django_pages.feed.admin import FeedAdmin
from django_pages.language.admin import LanguageAdmin
from django_pages.log.admin import LogAdmin
from django_pages.menu.admin import MenuItemAdmin
from django_pages.metadata.admin import MetaSetAdmin, MetaDataAdmin
from django_pages.pages.admin import PageAdmin, PostAdmin
from django_pages.site.admin import SiteAdmin

admin.site.register(Comment, CommentAdmin)
admin.site.register(FeedSettings, FeedAdmin)
admin.site.register(Language, LanguageAdmin)
admin.site.register(Log, LogAdmin)
admin.site.register(MenuItem, MenuItemAdmin)
admin.site.register(MetaSet, MetaSetAdmin)
admin.site.register(MetaData, MetaDataAdmin)
admin.site.register(Page, PageAdmin)
admin.site.register(Post, PostAdmin)
admin.site.register(Site, SiteAdmin)



import os

from django_pages.connector import ElementTree
from django_pages.connector import settings

from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt

import actions
import support
from support import actual_path, actual_url


@csrf_exempt
def browser(request):

    if request.user.is_staff:

        # extract the command, type and folder path
        command_name = request.REQUEST.get('Command', None)

        resource_type, resource_type_path = \
            support.get_resource_type_folder(request)

        folder_path = request.REQUEST.get('CurrentFolder', None)

        # set the default return values to successful completion
        err_no = 0
        err_txt = 'Successful'

        xml_response = ElementTree.ElementTree(
            ElementTree.Element("Connector", {'command': command_name,
                                              'resourceType': resource_type})
        )

        if None in (command_name, resource_type, folder_path):
            err_no = 1
            err_txt = 'Incomplete command.'
        else:
            # construct the response
            # append current folder information
            abs_path = actual_path(settings.FCKEDITOR_CONNECTOR_ROOT,
                                 resource_type_path, folder_path)
            abs_url = actual_url(settings.FCKEDITOR_CONNECTOR_URL,
                                 resource_type_path, folder_path)

            xml_response.getroot().append(
                ElementTree.Element("CurrentFolder",
                                    {'path': folder_path,
                                     'url': abs_url,
                                    }
                )
            )

            if (command_name == 'GetFolders'):
                # append Folder list
                actions.get_folders(xml_response, abs_path)

            elif (command_name == 'GetFoldersAndFiles'):
                # append Folder and list
                actions.get_folders_and_files(xml_response, abs_path)

            elif (command_name == 'CreateFolder'):
                new_folder_name = request.REQUEST.get('NewFolderName', None)

                if new_folder_name is None:
                    err_no = 102
                    err_txt = 'Invalid folder name.'
                else:

                    try:
                        os.mkdir(os.path.join(abs_path, new_folder_name))
                    except Exception, e:
                        err_no = 110
                        err_txt = str(e)

            elif (command_name == 'FileUpload'):

                status, filename = actions.file_upload(request, abs_path)

                if int(status) == 201:
                    # upload through the browser requires a conjoined
                    # status and filename if the file was renamed
                    status = "%s, '%s'" % (status, filename)

                return HttpResponse(
                    """<script type="text/javascript">
                    window.parent.frames['frmUpload'].OnUploadCompleted(%s)
                    </script>""" % status)

            else:
                # unknown command
                xml_response.getroot().append(
                    ElementTree.Element("Error", {'number': 1, 'text': 'blarf'})
                )

        xml_response.getroot().append(
            ElementTree.Element("Error", {'number': str(err_no), 'text': err_txt})
        )

        response = HttpResponse(ElementTree.tostring(xml_response.getroot(),
                                                    'utf-8'),
                                mimetype='text/xml')
        response['Cache-Control'] = 'no-cache'

        return response

    else:

        return None


def uploader(request):
    """Quick Uploader server-side support.  Responds to a POST request
    with the file uploaded as NewFile."""

    if request.user.is_staff:

        # extract the command, type and folder path
        try:
            resource_type, resource_type_path = \
                support.get_resource_type_folder(request)
            if resource_type is None:
                # no type specified
                resource_type_path = ''

            # determine the actual folder path
            folder_path = actual_path(settings.FCKEDITOR_CONNECTOR_ROOT,
                                    resource_type_path, '')

            # handle the upload
            status, filename = actions.file_upload(request, folder_path)

            # calculate the resulting file URL
            file_url = actual_url(settings.FCKEDITOR_CONNECTOR_URL,
                                resource_type_path, '', filename)

        except Exception, e:
            print e
            raise e

        # return an HTML response
        return HttpResponse(
            """<script type="text/javascript">
            window.parent.OnUploadCompleted(%s, '%s', '%s', '')

            </script>""" % (status, file_url, filename,)
        )

    else:

        return None



# -*- encoding: utf-8 -*-

from django.conf import settings


ADMIN_MEDIA_PREFIX = '/static/admin/'

# where to upload flag images /media/FLAG_UPLOAD_DIR
FLAG_UPLOAD_DIR = settings.MEDIA_ROOT

# where to look for template in templates/
TEMPLATE_PATH = 'default'

POSTS_ON_PAGE = 10



"""

==========================
FCKeditor Django Connector
==========================

 :Author: Nathan R. Yergler <nathan@yergler.net>
 :Copyright: 2006-2007
 :License: GNU LGPL; see LICENSE for details.
 :Version: $Rev$
 :Updated: $Date: 2006-08-17 11:57:00 -0400 (Thu, 17 Aug 2006) $

.. contents:: Document Index
    :class: docindex

Overview
--------

 FCKeditor_ is a Javascript-based rich text editor for web applications.  One
 of the enhanced features it offers is a web-based browser for media
 files stored on the server.  The browser relies on a server-side
 *connector* to provide XML responses for commands.  FCKeditor includes
 connectors for many server-side technologies, including a Python CGI
 implementation.  However, when using FCKeditor in a Django_ application,
 it is desirable to implement the connector within the Django process.  This
 package provides a Django-based implementation of the FCKeditor
 `server side specification`_.

Dependencies
------------

 The FCKeditor Connector relies on Django and ElementTree_.  Note that if
 you prefer to use lxml_, you can simply modify the import at the top
 of ``views.py`` from::

   from elementtree import ElementTree

 to::

   import lxml.etree as ElementTree

 
Usage
-----

 To use the FCKeditor Connector you must install the connector as a Django
 application in your project, connect the appropriate URLs, modify the
 admin interface to use FCKeditor and finally update the FCKeditor
 configuration to point to the connector.

Installing the Application
++++++++++++++++++++++++++

 To install the application, simply place the ``fckeditor_connector``
 package on your PYTHONPATH and add ``fckeditor_connector`` to the
 ``INSTALLED_APPS`` setting in ``settings.py``.

 After installing the application you need to set two configuration
 variables found in ``views.py``.  ``BASE_PATH`` defines the base
 file-system path for FCKeditor-browsable files.  Note that FCKeditor
 expects that this directory will contain sub-directories for specific
 file types.  For example, if ``BASE_PATH`` is set to ``/var/www/media``,
 FCKeditor expects that ``/var/www/media/Images`` will contain image files.
 See the `server side specification`_ for details on FCKeditor file paths.

 ``BASE_URL`` should contain the base URL for the files served.  This has
 only been tested with files served from the same server URL as Django
 (using the `static files` Django view for development).

Connecting the URLs
+++++++++++++++++++

 To enable the FCKeditor Connector URLs in your project, you can add
 something like::

    ...
    
    (r'^fckeditor_connector/', include('fckeditor.connector.urls')),

    ...

 to your project ``urls.py``.  If you want to use a different prefix, you'll
 need to use that instead of ``fckeditor_connector`` when configuring
 FCKeditor below.
 
Configuring FCKeditor
+++++++++++++++++++++

 To enable FCKeditor to use the Django connector, you need to update
 ``fckconfig.js`` to point to the connector.  The default FCKeditor
 connector configuration is based on a standard URL schema.  Unfortunately
 this does not work with the Django connector, so we have to specify a
 complete URL.  In ``fckconfig.js`` you will find a block on configuration
 parameters such as::
 
   FCKConfig.ImageBrowser = true ;
   FCKConfig.ImageBrowserURL = FCKConfig.BasePath + 'filemanager/browser/default/browser.html?Type=Image&Connector=connectors/' + _FileBrowserLanguage + '/connector.' + _FileBrowserExtension ;
   FCKConfig.ImageBrowserWindowWidth  = FCKConfig.ScreenWidth * 0.7 ;	// 70% ;
   FCKConfig.ImageBrowserWindowHeight = FCKConfig.ScreenHeight * 0.7 ;	// 70% ;

 Below this block add the following line::
 
   FCKConfig.ImageBrowserURL = FCKConfig.BasePath + 'filemanager/browser/default/browser.html?Type=Image&Connector=/fckeditor_connector/browser/';

 Note that if you did not use the suggested URL prefix you will need to
 modify this line.  In particular, the first portion of the ``Connector``
 query string parameter should contain the base URL provided in your project's
 ``urls.py``.

Using FCKeditor in the Admin interface
++++++++++++++++++++++++++++++++++++++

 There are several ways to connect the FCKeditor to the Admin interface.  The
 way we have tested with is to replace a specific ``textarea``.  To use the
 FCKeditor, you need to download FCKeditor and put it in a location
 accessible by Django.  Currently we put this underneath the admin
 media directory.  Once you have it accessible, you need to add the following
 information to the ``Admin`` class of the appropriate model::

     class Admin:
        # javascript for fck editor
        js = ( 'underlay/fckeditor/fckeditor.js',
               'underlay/news/story.js',
               )


 In this case ``story.js`` is a small Javascript file which contains the
 following::

     window.onload = function()
     {
         // create the FCK Editor and replace the specific text area with it
         var oFCKeditor = new FCKeditor( 'id_text' ) ;
         oFCKeditor.BasePath	= '/underlay/fckeditor/'; 
         oFCKeditor.ReplaceTextarea() ;
     }

 Note that ``id_text`` is the name of the TextField in your model prepended
 with ``id``.  The ``BasePath`` setting should reflect the absolute path you
 used to make the FCKeditor files available.

Known Issues
------------

 The Connector currently only implements File Browser Connector at this point.
 The `Quick Uploader`_ API will be implemented in the future.

 Permission problems are not correctly reported.

Contact Information
-------------------


.. _FCKeditor: http://www.fckeditor.net
.. _Django: http://djangoproject.com
.. _`server side specification`: http://wiki.fckeditor.net/Developer%27s_Guide/Participating/Server_Side_Integration#Browser
.. _ElementTree: http://effbot.org/zone/element-index.htm
.. _lxml: http://codespeak.net/lxml
.. _`static files`: http://www.djangoproject.com/documentation/static_files/
.. _`Quick Uploader`: http://wiki.fckeditor.net/Developer%27s_Guide/Participating/Server_Side_Integration#Upload

"""


try:
    from elementtree import ElementTree
except ImportError:
    try:
        import lxml.etree as ElementTree
    except ImportError:
        raise ImportError, "Neither elementtree.ElementTree nor lxml.etree was found. This is required by fckeditor_connector"



# -*- encoding: utf-8 -*-

from django.contrib import admin

import reversion

from django_pages.metadata.models import MetaData


class MetaDataInline(admin.TabularInline):

    model = MetaData


class MetaSetAdmin(reversion.VersionAdmin):

    fields = (('language', 'name'),)
    list_display = ('name', 'language')
    inlines = (MetaDataInline, )


class MetaDataAdmin(reversion.VersionAdmin):

    list_display = ('__unicode__', 'name', 'content')



# -*- encoding: utf-8 -*-


def get_metadata(page):
    '''
    returns metadata for page

    @param page: Page object
    @return MetaData QuerySet
    '''

    if page.metadata_set:

        return page.metadata_set.metadata_set.all()

    return None



import datetime

from django.utils.timezone import make_aware, get_current_timezone

from django_pages.log.models import Log


class RequestLog(object):

    def process_request(self, request):

        now = make_aware(datetime.datetime.now(), get_current_timezone())
        log = Log(when=now, logline=str(request))
        log.save()
